/**
 * Abstract hierarchical factory class
 */
export class CustomFactory extends BaseFactory {
    /**
     * The Root Factory name
     * @abstract
     * @optional
     * @type {string}
     */
    static ROOT_NAME: string;
    static getClassList(ctor: any): any[];
    static getClassNameList(ctor: any): any[];
    /**
     * @internal
     * @param {*} aClass
     * @param {*|number} [aParentClass]
     * @param {number} [aBaseNameOnly]
     * @returns
     */
    static formatNameFromClass(aClass: any, aParentClass?: any | number, aBaseNameOnly?: number): string;
    /**
     * get path of a class or itself
     * @param {*=} aClass
     * @param {*=} aRootName
     * @returns {string[]}
     */
    static path(aClass?: any | undefined, aRootName?: any | undefined): string[];
    /**
     * get path array of a class or itself
     * @param {*=} aClass
     * @param {*=} aRootName
     * @returns {string[]}
     */
    static pathArray(aClass?: any | undefined, aRootName?: any | undefined): string[];
    static _registerWithParent(aClass: any, aParentClass: any, aOptions: any): any;
    /**
     * register the aClass to the factory
     * @param {typeof CustomFactory=} aClass the class to register the Factory
     * @param {typeof CustomFactory|ICustomFactoryOptions=} aParentClass the optional parent class
     * @param {ICustomFactoryOptions|any=} aOptions the options for the class and the factory
     * @returns {boolean} return true if successful.
     */
    static register(aClass?: typeof CustomFactory | undefined, aParentClass?: (typeof CustomFactory | ICustomFactoryOptions) | undefined, aOptions?: (ICustomFactoryOptions | any) | undefined): boolean;
}
export type ICustomFactoryOptions = import('./base-factory.js').IBaseFactoryOptions & {
    baseNameOnly?: number;
    parent?: typeof CustomFactory;
};
import { BaseFactory } from './base-factory.js';
